package io.useless.play.authentication

import java.util.UUID
import scala.util.{ Success, Failure }
import scala.concurrent.{ future, Future, Await }
import scala.concurrent.ExecutionContext.Implicits.global
import scala.concurrent.duration._

import play.api.mvc._

import io.useless.auth.Account
import io.useless.util.Uuid

trait AuthenticatorComponent {

  val authenticator: Authenticator

  trait Authenticator {

    def authenticate[A](request: Request[A]): Future[Option[Account]]

  }

}

trait CompositeAuthenticatorComponent extends AuthenticatorComponent {

  class CompositeAuthenticator(
    authenticators: Seq[Authenticator]
  ) extends Authenticator {

    def authenticate[A](request: Request[A]): Future[Option[Account]] = {
      future {
        var result: Option[Account] = None

        for (authenticator <- authenticators) {
          if (result.isEmpty) {
            result = Await.result(
              authenticator.authenticate(request),
              100.millis
            )
          }
        }

        result
      }
    }

  }

}

trait GuidAuthenticatorComponent extends AuthenticatorComponent {

  self: AuthDaoComponent =>

  trait GuidAuthenticator extends Authenticator {

    def guid[A](request: Request[A]): Option[String]

    def authenticate[A](request: Request[A]) = {
      guid(request).map{ rawGuid =>
        Uuid.parseUuid(rawGuid) match {
          case Success(guid: UUID) => {
            authDao.getAccessToken(guid).map { maybeAccessToken =>
              maybeAccessToken.map(_.resourceOwner)
            }
          }
          case _: Failure[UUID] => future { None }
        }
      }.getOrElse { future { None } }
    }

  }

}

trait HeaderAuthenticatorComponent extends GuidAuthenticatorComponent {

  self: AuthDaoComponent =>

  class HeaderAuthenticator(header: String) extends GuidAuthenticator {

    def guid[A](request: Request[A]) = request.headers.get(header)

  }

}

trait QueryParameterAuthenticatorComponent extends GuidAuthenticatorComponent {

  self: AuthDaoComponent =>

  class QueryParameterAuthenticator(queryParameter: String) extends GuidAuthenticator {

    def guid[A](request: Request[A]) = request.getQueryString(queryParameter)

  }

}

trait CookieAuthenticatorComponent extends GuidAuthenticatorComponent {

  self: AuthDaoComponent =>

  class CookieAuthenticator(cookie: String) extends GuidAuthenticator {

    def guid[A](request: Request[A]) = request.cookies.get(cookie).map(_.value)

  }

}
